/*
 * Copyright (c) 2011, diablitozzz.org All rights reserved. Redistribution
 * and use in source and binary forms, with or without modification, are
 * permitted provided that the following conditions are met: * Redistributions
 * of source code must retain the above copyright notice, this list of
 * conditions and the following disclaimer. * Redistributions in binary form
 * must reproduce the above copyright notice, this list of conditions and the
 * following disclaimer in the documentation and/or other materials provided
 * with the distribution. * Neither the name of diablitozzz.org nor the names of
 * its contributors may be used to endorse or promote products derived from this
 * software without specific prior written permission. THIS SOFTWARE IS PROVIDED
 * BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.diablitozzz.jera.jetty.app;

import java.util.Map;

import org.apache.commons.daemon.Daemon;
import org.apache.commons.daemon.DaemonContext;
import org.diablitozzz.jera.jetty.build.BuildServletHandler;
import org.diablitozzz.jera.jetty.config.ConfigBuild;
import org.diablitozzz.jera.jetty.config.ConfigHost;
import org.diablitozzz.jera.jetty.config.ConfigHosts;
import org.diablitozzz.jera.jetty.config.ConfigServer;
import org.diablitozzz.jera.jetty.config.ConfigService;
import org.diablitozzz.jera.jetty.error.ErrHandle;
import org.eclipse.jetty.server.Connector;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.handler.HandlerList;
import org.eclipse.jetty.server.nio.SelectChannelConnector;
import org.eclipse.jetty.webapp.WebAppContext;

public class Daemonz implements Daemon {

    private String configPath;
    private final ConfigService configService = new ConfigService();

    private Server server;

    @Override
    public void destroy()
    {
        this.server = null;
    }

    public String getConfigPath()
    {
        return this.configPath;
    }

    @Override
    public void init(DaemonContext context) throws Exception
    {
        this.setConfigPath(context.getArguments()[0]);
    }

    public void setConfigPath(String configPath)
    {
        this.configPath = configPath;
    }

    @Override
    public void start() throws Exception
    {

        this.server = new Server();

        ConfigServer serverConfig = this.configService.loadConfig(this.getConfigPath());

        // create loggers
        if (serverConfig.getOut() != null) {
            System.setOut(serverConfig.getOutAsStream());
        }
        if (serverConfig.getErr() != null) {
            System.setErr(serverConfig.getErrAsStream());
        }

        // config connector
        final Connector connector = new SelectChannelConnector();
        connector.setPort(serverConfig.getPort());
        this.server.setConnectors(new Connector[] { connector });

        // make hosts
        HandlerList handlers = new HandlerList();
        ConfigHosts hosts = serverConfig.getHosts();

        for (ConfigHost host : hosts.getHostList()) {

            WebAppContext context = new WebAppContext();

            context.setResourceBase(host.getRoot());
            context.setVirtualHosts(new String[] { host.getName() });
            context.setErrorHandler(new ErrHandle());

            // make build
            ConfigBuild build = host.getBuild();
            if (build != null) {
                context.setServletHandler(new BuildServletHandler(build));
            }

            // make params
            Map<String, String> params = host.getParams();
            if (params != null) {

                for (String paramName : params.keySet()) {
                    context.setInitParameter(paramName, params.get(paramName));
                }
            }

            handlers.addHandler(context);
        }

        this.server.setHandler(handlers);

        this.server.start();
        this.server.join();
    }

    @Override
    public void stop() throws Exception
    {
        this.server.stop();
        this.server = null;
    }

}
