/*
 * Copyright (c) 2011, diablitozzz.org All rights reserved. Redistribution
 * and use in source and binary forms, with or without modification, are
 * permitted provided that the following conditions are met: * Redistributions
 * of source code must retain the above copyright notice, this list of
 * conditions and the following disclaimer. * Redistributions in binary form
 * must reproduce the above copyright notice, this list of conditions and the
 * following disclaimer in the documentation and/or other materials provided
 * with the distribution. * Neither the name of diablitozzz.org nor the names of
 * its contributors may be used to endorse or promote products derived from this
 * software without specific prior written permission. THIS SOFTWARE IS PROVIDED
 * BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.diablitozzz.jera.jetty.build;

import java.io.IOException;

import javax.servlet.Servlet;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.diablitozzz.jera.jetty.ant.AntErrorException;
import org.diablitozzz.jera.jetty.ant.AntRunner;
import org.diablitozzz.jera.jetty.compile.Loader;
import org.diablitozzz.jera.jetty.compile.ScannerService;
import org.diablitozzz.jera.jetty.config.ConfigBuild;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class BuildService {

    private final AntRunner antRunner = new AntRunner();
    private final ConfigBuild buildConfig;

    private final Logger logger = LoggerFactory.getLogger(BuildService.class);
    private Servlet original;
    private final String originalClassName;

    private final ScannerService scannerService;

    public BuildService(String originalClassName, ConfigBuild buildConfig) {
        this.buildConfig = buildConfig;
        this.originalClassName = originalClassName;
        this.scannerService = new ScannerService(buildConfig.getScan());
    }

    private void compile() throws AntErrorException
    {
        this.logger.info("Run ant [{}]", this.buildConfig.getAnt());
        this.antRunner.setBuildFile(this.buildConfig.getAnt());
        this.antRunner.start();
    }

    public void destroy()
    {
        if (this.original != null) {
            this.original.destroy();
        }
        this.original = null;
    }

    private void init(ServletConfig config) throws ServletException,
            InstantiationException, IllegalAccessException,
            ClassNotFoundException
    {

        final String className = this.originalClassName;
        this.logger.info("Create instance [{}]", className);

        final HttpServlet newServlet = (HttpServlet) Thread.currentThread()
                .getContextClassLoader().loadClass(className).newInstance();

        this.original = newServlet;
        this.original.init(config);
    }

    public void iteration(ServletConfig config, HttpServletRequest req,
            HttpServletResponse res) throws ServletException, IOException,
            ClassNotFoundException, InstantiationException,
            IllegalAccessException, AntErrorException
    {

        if (!this.scannerService.isEmpty()) {
            if (this.scannerService.isChange() || (this.original == null)) {
                this.destroy();
                this.compile();
                this.reload();
                this.scannerService.reScan();
            }
        }

        if (this.original == null) {
            this.init(config);
        }

        this.original.service(req, res);
    }

    private void reload() throws ClassNotFoundException,
            InstantiationException, IllegalAccessException
    {
        final String classesDir = this.buildConfig.getClasses();
        this.logger.info("Switch class loader [{}]", classesDir);
        final Loader loader = new Loader(Thread.currentThread()
                .getContextClassLoader());
        loader.setSearchDir(classesDir);
        Thread.currentThread().setContextClassLoader(loader);
    }

}
