/*
* Copyright (c) 2011, diablitozzz.org All rights reserved. Redistribution
* and use in source and binary forms, with or without modification, are
* permitted provided that the following conditions are met: * Redistributions
* of source code must retain the above copyright notice, this list of
* conditions and the following disclaimer. * Redistributions in binary form
* must reproduce the above copyright notice, this list of conditions and the
* following disclaimer in the documentation and/or other materials provided
* with the distribution. * Neither the name of diablitozzz.org nor the names of
* its contributors may be used to endorse or promote products derived from this
* software without specific prior written permission. THIS SOFTWARE IS PROVIDED
* BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
* EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS;
* OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.diablitozzz.jera.jetty.compile;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;

public class Loader extends ClassLoader {

    @SuppressWarnings("rawtypes")
    private final HashMap<String, Class> classes = new HashMap<String, Class>();

    private String searchDir;

    public Loader(ClassLoader parent) {
        super(parent);
    }

    public Loader(Loader parent) {
        super(parent.getParent());
    }

    private String classNameToFileName(String className) {
        return className.replace('.', '/') + ".class";
    }

    @Override
    protected Class<?> findClass(String name) throws ClassNotFoundException {

        @SuppressWarnings("rawtypes")
        final Class c = this.classes.get(name);
        if (c == null) {
            return super.findClass(name);
        }
        return c;
    }

    private byte[] getBytesFromFile(File file) throws IOException {
        final long len = file.length();
        final byte raw[] = new byte[(int) len];
        final FileInputStream fin = new FileInputStream(file);
        final int r = fin.read(raw);
        if (r != len) {
            throw new IOException("Can't read all, " + r + "!=" + len);
        }
        fin.close();
        return raw;
    }

    public String getSearchDir() {
        return this.searchDir;
    }

    @Override
    public Class<?> loadClass(String className, boolean resolve) throws ClassNotFoundException {

        @SuppressWarnings("rawtypes")
        Class c = null;

        // load class from cache
        c = this.classes.get(className);
        if (c != null) {
            return c;
        }

        // load class from file
        c = this.loadClassFromFile(className);

        // load class from super
        if (c == null) {
            c = super.loadClass(className, resolve);
        }

        // if class found
        if (c != null) {
            if (resolve) {
                this.resolveClass(c);
            }
            this.classes.put(className, c);
        }
        return c;
    }

    @SuppressWarnings({ "rawtypes" })
    private Class loadClassFromFile(String className) throws ClassNotFoundException {
        String fileName = this.classNameToFileName(className);
        fileName = this.getSearchDir() + File.separator + fileName;

        final File file = new File(fileName).getAbsoluteFile();
        if (!file.exists()) {
            return null;
        }
        try {
            final byte[] data = this.getBytesFromFile(file);
            return this.defineClass(className, data, 0, data.length);
        } catch (final IOException e) {
            return null;
        }
    }

    public void setSearchDir(String searchDir) {
        this.searchDir = searchDir;
    }

}
