/*
* Copyright (c) 2011, diablitozzz.org All rights reserved. Redistribution
* and use in source and binary forms, with or without modification, are
* permitted provided that the following conditions are met: * Redistributions
* of source code must retain the above copyright notice, this list of
* conditions and the following disclaimer. * Redistributions in binary form
* must reproduce the above copyright notice, this list of conditions and the
* following disclaimer in the documentation and/or other materials provided
* with the distribution. * Neither the name of diablitozzz.org nor the names of
* its contributors may be used to endorse or promote products derived from this
* software without specific prior written permission. THIS SOFTWARE IS PROVIDED
* BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
* EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS;
* OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.diablitozzz.jera.jetty.compile;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

class ScanItem {

    private File file;
    private long lastMod;
    private String path;

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (!(obj instanceof ScanItem)) {
            return false;
        }
        final ScanItem scanItem = (ScanItem) obj;
        if (!this.getPath().equals(scanItem.getPath())) {
            return false;
        }
        return this.getLastMod() == scanItem.getLastMod();
    }

    public File getFile() {
        return this.file;
    }

    public long getLastMod() {
        return this.lastMod;
    }

    public String getPath() {
        return this.path;
    }

    public void setFile(File file) {
        this.path = file.getAbsolutePath();
        this.lastMod = file.lastModified();
        this.file = file;
    }

}

public class Scanner {

    private final List<ScanItem> files = new ArrayList<ScanItem>();
    private String root;

    public boolean equals(Scanner scanner) {
        return this.getFiles().equals(scanner.getFiles());
    }

    public List<ScanItem> getFiles() {
        return this.files;
    }

    public String getRoot() {
        return this.root;
    }

    public void scan() {
        this.files.clear();
        final File root = new File(this.root);

        if (root.exists()) {
            this.scanRecursively(root);
        }
    }

    private void scanRecursively(File file) {

        final ScanItem scanItem = new ScanItem();
        scanItem.setFile(file);
        this.files.add(scanItem);

        // get sub files
        final File[] children = file.listFiles();
        if (children == null) {
            return;
        }
        for (final File child : children) {
            this.scanRecursively(child);
        }
    }

    public void setRoot(String root) {
        this.root = root;
    }

}