/*
 * Copyright (c) 2011, diablitozzz.org All import org.apache.commons.digester.annotations.rules.ObjectCreate;
 * import org.apache.commons.digester.annotations.rules.SetProperty;
 * following conditions are met: * Redistributions
 * of source code must retain the above copyright notice, this list of
 * conditions and the following disclaimer. * Redistributions in binary form
 * must reproduce the above copyright notice, this list of conditions and the
 * following disclaimer in the documentation and/or other materials provided
 * with the distribution. * Neither the name of diablitozzz.org nor the names of
 * its contributors may be used to endorse or promote products derived from this
 * software without specific prior written permission. THIS SOFTWARE IS PROVIDED
 * BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.diablitozzz.jera.jetty.config;

import java.util.HashMap;
import java.util.Map;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class ConfigHost {

    private ConfigBuild build;
    private String name;
    private Map<String, String> params;
    private String root;

    public ConfigBuild getBuild()
    {
        return this.build;
    }

    public String getName()
    {
        return this.name;
    }

    public Map<String, String> getParams()
    {
        return this.params;
    }

    public String getRoot()
    {
        return this.root;
    }

    public void load(Node node, ConfigUtils configUtils) throws XPathExpressionException
    {
        // find host
        XPath xpath = configUtils.getXPath();
        XPathExpression expr = xpath.compile("//host[1]");
        Node host = (Node) expr.evaluate(node, XPathConstants.NODE);

        // find attrs
        this.name = configUtils.makeStringValue(host, "name");
        this.root = configUtils.makeStringValue(host, "root");

        // find build
        expr = xpath.compile("//build[1]");
        Node build = (Node) expr.evaluate(host, XPathConstants.NODE);
        if (build != null) {
            this.build = new ConfigBuild();
            this.build.load(build, configUtils);
        }

        // find params
        expr = xpath.compile("//params[1]");
        Node params = (Node) expr.evaluate(host, XPathConstants.NODE);
        if (params != null) {
            this.makeParams(params, configUtils);
        }
    }

    private void makeParams(Node params, ConfigUtils configUtils) throws XPathExpressionException
    {
        this.params = new HashMap<String, String>();
        // find nodes
        XPath xpath = configUtils.getXPath();
        XPathExpression expr = xpath.compile("//param");

        NodeList paramList = (NodeList) expr.evaluate(params, XPathConstants.NODESET);

        for (int i = 0; i < paramList.getLength(); i++) {

            Node param = paramList.item(i);
            String name = configUtils.makeStringValue(param, "name");
            String value = configUtils.makeStringValue(param, "value");
            this.params.put(name, value);
        }

    }

}
