/*
 * Copyright (c) 2011, diablitozzz.org All import org.apache.commons.digester.annotations.rules.ObjectCreate;
 * import org.apache.commons.digester.annotations.rules.SetProperty;
 * following conditions are met: * Redistributions
 * of source code must retain the above copyright notice, this list of
 * conditions and the following disclaimer. * Redistributions in binary form
 * must reproduce the above copyright notice, this list of conditions and the
 * following disclaimer in the documentation and/or other materials provided
 * with the distribution. * Neither the name of diablitozzz.org nor the names of
 * its contributors may be used to endorse or promote products derived from this
 * software without specific prior written permission. THIS SOFTWARE IS PROVIDED
 * BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.diablitozzz.jera.jetty.config;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;

import org.w3c.dom.Node;
import org.xml.sax.SAXException;

public class ConfigServer {

    private String err;
    private ConfigHosts hosts;
    private String out;
    private int port;

    public String getErr()
    {
        return this.err;
    }

    public PrintStream getErrAsStream() throws FileNotFoundException
    {
        String file = this.getErr();
        if (file != null) {
            OutputStream output = new FileOutputStream(file, true);
            return new PrintStream(output);
        }
        return null;
    }

    public ConfigHosts getHosts()
    {
        return this.hosts;
    }

    public String getOut()
    {
        return this.out;
    }

    public PrintStream getOutAsStream() throws FileNotFoundException
    {
        String file = this.getOut();
        if (file != null) {
            OutputStream output = new FileOutputStream(file, true);
            return new PrintStream(output);
        }
        return null;
    }

    public int getPort()
    {
        return this.port;
    }

    public void load(Node node, ConfigUtils configUtils) throws XPathExpressionException, ParserConfigurationException, SAXException, IOException
    {
        XPath xpath = configUtils.getXPath();

        // find server
        XPathExpression expr = xpath.compile("//server[1]");
        Object result = expr.evaluate(node, XPathConstants.NODE);
        Node server = (Node) result;

        this.port = configUtils.makeIntValue(server, "port");
        this.err = configUtils.makeStringValue(server, "err");
        this.out = configUtils.makeStringValue(server, "out");

        // make hosts
        this.hosts = new ConfigHosts();
        this.hosts.load(server, configUtils);
    }

}
