/*
 * Copyright (c) 2011, diablitozzz.org All import org.apache.commons.digester.annotations.rules.ObjectCreate;
 * import org.apache.commons.digester.annotations.rules.SetProperty;
 * following conditions are met: * Redistributions
 * of source code must retain the above copyright notice, this list of
 * conditions and the following disclaimer. * Redistributions in binary form
 * must reproduce the above copyright notice, this list of conditions and the
 * following disclaimer in the documentation and/or other materials provided
 * with the distribution. * Neither the name of diablitozzz.org nor the names of
 * its contributors may be used to endorse or promote products derived from this
 * software without specific prior written permission. THIS SOFTWARE IS PROVIDED
 * BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.diablitozzz.jera.jetty.config;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

public class ConfigUtils {

    private final DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
    private Map<String, String> env = new HashMap<String, String>();
    private final XPath xpath = XPathFactory.newInstance().newXPath();

    public void clear()
    {
        this.env.clear();
    }

    public Map<String, String> getEnv()
    {
        return this.env;
    }

    public XPath getXPath()
    {
        return this.xpath;
    }

    public Document loadFile(File file) throws ParserConfigurationException, SAXException, IOException
    {
        DocumentBuilder db = this.documentBuilderFactory.newDocumentBuilder();
        Document document = db.parse(file);
        return document;
    }

    public int makeIntValue(Node node, String attrName)
    {
        String value = this.makeStringValue(node, attrName);
        return Integer.parseInt(value);
    }

    public String makeStringValue(Node node, String attrName)
    {
        String value = node.getAttributes().getNamedItem(attrName).getNodeValue();
        value = this.parseEnv(value);
        return value;
    }

    public String parseEnv(String value)
    {
        for (String k : this.env.keySet()) {
            String v = this.env.get(k);
            value = value.replace("${" + k + "}", v);
        }
        return value;
    }
}
