/*
 * Copyright (c) 2011, diablitozzz.org All rights reserved. Redistribution
 * and use in source and binary forms, with or without modification, are
 * permitted provided that the following conditions are met: * Redistributions
 * of source code must retain the above copyright notice, this list of
 * conditions and the following disclaimer. * Redistributions in binary form
 * must reproduce the above copyright notice, this list of conditions and the
 * following disclaimer in the documentation and/or other materials provided
 * with the distribution. * Neither the name of diablitozzz.org nor the names of
 * its contributors may be used to endorse or promote products derived from this
 * software without specific prior written permission. THIS SOFTWARE IS PROVIDED
 * BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.diablitozzz.jera.jetty.error;

import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.diablitozzz.jera.jetty.ant.AntErrorException;
import org.eclipse.jetty.server.HttpConnection;
import org.eclipse.jetty.server.Request;
import org.eclipse.jetty.server.handler.ErrorHandler;

import com.thoughtworks.xstream.XStream;

public class ErrHandle extends ErrorHandler {

    private final static String debugExceptionMethod = "getJeraJettyDebugObject";

    private Object getDebugExceptionData(Throwable e) throws NoSuchMethodException, SecurityException, IllegalAccessException, IllegalArgumentException,
            InvocationTargetException
    {
        Method method = e.getClass().getMethod(ErrHandle.debugExceptionMethod);
        method.setAccessible(true);
        return method.invoke(e);
    }

    @Override
    public void handle(String target, Request baseRequest, HttpServletRequest request, HttpServletResponse response) throws IOException
    {
        try {
            this.run(target, baseRequest, request, response);
        } catch (final IOException e) {
            throw e;
        } catch (final Throwable e) {

            throw new IOException(e);
        }
    }

    private boolean isDebugException(Throwable e)
    {
        if (e == null) {
            return false;
        }

        if (!(e instanceof RuntimeException)) {
            return false;
        }

        for (Method method : e.getClass().getMethods())
        {
            if (method.getName().equals(ErrHandle.debugExceptionMethod)) {
                return true;
            }
        }

        return true;
    }

    private void run(String target, Request baseRequest, HttpServletRequest request, HttpServletResponse response) throws Throwable
    {

        HttpConnection connection = HttpConnection.getCurrentConnection();
        connection.getRequest().setHandled(true);

        Throwable exception = (Throwable) request.getAttribute("javax.servlet.error.exception");

        // debug exception
        if (this.isDebugException(exception)) {

            response.setContentType("text/xml");
            response.setCharacterEncoding("utf-8");

            XStream xstream = new XStream();
            Writer writer = new OutputStreamWriter(response.getOutputStream(), "UTF-8");
            writer.write("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>");
            xstream.toXML(this.getDebugExceptionData(exception), writer);
            return;
        }

        response.setCharacterEncoding("utf-8");
        response.setContentType("text/html");

        final Printer printer = new Printer();
        printer.setWriter(response.getWriter());
        printer.printHeader(connection.getResponse().getReason());

        if (exception != null) {
            printer.printException(exception);
            if (exception instanceof AntErrorException) {
                printer.printAntErrorException((AntErrorException) exception);
            }
            printer.printStackTrace(exception);
        }

        printer.printServletContext(baseRequest.getServletContext());
        printer.printMemory();
        printer.printSystemEnvironment();
        printer.printSystemProperties();
        printer.printInfo();
        printer.printFooter();

    }

}
