/*
* Copyright (c) 2011, diablitozzz.org All rights reserved. Redistribution
* and use in source and binary forms, with or without modification, are
* permitted provided that the following conditions are met: * Redistributions
* of source code must retain the above copyright notice, this list of
* conditions and the following disclaimer. * Redistributions in binary form
* must reproduce the above copyright notice, this list of conditions and the
* following disclaimer in the documentation and/or other materials provided
* with the distribution. * Neither the name of diablitozzz.org nor the names of
* its contributors may be used to endorse or promote products derived from this
* software without specific prior written permission. THIS SOFTWARE IS PROVIDED
* BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
* EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS;
* OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.diablitozzz.jera.jetty.error;

import java.io.PrintWriter;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;

import org.apache.commons.lang.StringEscapeUtils;
import org.apache.tools.ant.BuildEvent;
import org.apache.tools.ant.Target;
import org.apache.tools.ant.Task;
import org.diablitozzz.jera.jetty.ant.AntErrorException;

public class Printer {

    private PrintWriter writer;

    public void printAntErrorException(AntErrorException e) {

        String key, value, title;
        final BuildEvent event = e.getEvent();

        title = "Ant Build Event";
        this.writer.println("<div class=\"list exception\">");
        this.writer.println("<h2>" + title + "</h2>");

        this.writer.println("<ul>");

        // Main
        this.writer.println("<li>");
        this.writer.println("<table>");

        // key = "System.out";
        // value = e.getOut();
        // this.writer.println("<tr><th>" + key + "</th><td>" + value +
        // "</td></tr>");

        // key = "System.err:";
        // value = e.getErr();
        // this.writer.println("<tr><th>" + key + "</th><td>" + value +
        // "</td></tr>");

        key = "Event message:";
        value = event.getMessage();
        this.writer.println("<tr><th>" + key + "</th><td>" + value + "</td></tr>");

        this.writer.println("</table>");
        this.writer.println("</li>");

        // project
        this.writer.println("<li>");
        this.writer.println("<table>");

        key = "Project Name:";
        value = event.getProject().getName();
        this.writer.println("<tr><th>" + key + "</th><td>" + value + "</td></tr>");

        this.writer.println("</table>");
        this.writer.println("</li>");

        // target
        final Target target = event.getTarget();

        if (target != null) {

            this.writer.println("<li>");
            this.writer.println("<table>");

            key = "Target Name:";
            value = target.getName();
            this.writer.println("<tr><th>" + key + "</th><td>" + value + "</td></tr>");

            key = "Target File:";
            value = target.getLocation().getFileName();
            this.writer.println("<tr><th>" + key + "</th><td>" + value + "</td></tr>");

            key = "Target Line:";
            value = new Integer(target.getLocation().getLineNumber()).toString();
            this.writer.println("<tr><th>" + key + "</th><td>" + value + "</td></tr>");

            this.writer.println("</table>");
            this.writer.println("</li>");
        }

        // Task
        final Task task = event.getTask();
        if (task != null) {
            this.writer.println("<li>");
            this.writer.println("<table>");

            key = "Task Name:";
            value = task.getTaskName();
            this.writer.println("<tr><th>" + key + "</th><td>" + value + "</td></tr>");

            key = "Task Type:";
            value = task.getTaskType();
            this.writer.println("<tr><th>" + key + "</th><td>" + value + "</td></tr>");

            key = "Task File:";
            value = task.getLocation().getFileName();
            this.writer.println("<tr><th>" + key + "</th><td>" + value + "</td></tr>");

            key = "Task Line:";
            value = new Integer(task.getLocation().getLineNumber()).toString();
            this.writer.println("<tr><th>" + key + "</th><td>" + value + "</td></tr>");

            this.writer.println("</table>");
            this.writer.println("</li>");
        }
        //
        //
        //
        this.writer.println("</table>");
        this.writer.println("</div>");
    }

    public void printException(Throwable e) {

        final Map<Object, Object> props = new HashMap<Object, Object>();

        props.put("Message", e.getMessage());
        props.put("Localized Message", e.getLocalizedMessage());
        props.put("Class", e.getClass().getCanonicalName());
        this.printProps(e.getClass().getCanonicalName(), props);
    }

    public void printFooter() {
        this.writer.println("</body>");
        this.writer.println("</html>");
    }

    public void printHeader(String title) {
        this.writer.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
        this.writer.println("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">");
        this.writer.println("<html xmlns=\"http://www.w3.org/1999/xhtml\" lang=\"ru\" xml:lang=\"ru\">");
        this.writer.println("<head>");
        this.writer.println("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />");
        this.writer.println("<title>" + title + "</title>");

        this.writer.println("<style>");

        this.writer.println("* { font-family: Arial, serif; font-size: 1em; }");

        this.writer.println("div.exception {width:100%;background-color:#eeeeee; border:solid 1px #770000;padding:0; margin: 0 0 0.5em 0;}");
        this.writer.println("div.exception > h2 {margin:0;background-color:#770000;color:#FFFFFF; font-weight:bold;padding:2px 2px 2px 4px;}");

        this.writer.println("div.key-value table { width:100%; border:none; border-spacing:0; border-collapse: collapse;}");
        this.writer.println("div.key-value table th, div.key-value table td {text-align:left;padding: 4px 4px 4px 4px;vertical-align:top;}");
        this.writer.println("div.key-value table th {width:15%;}");
        this.writer.println("div.key-value table tr:hover { background-color:#8fc8c9; }");

        this.writer.println("div.list ul { padding:0;margin:0;list-style: none;}");
        this.writer.println("div.list li { padding: 4px; margin: 0.3em; background-color:#cfe0e0; }");
        this.writer.println("div.list li:hover { background-color:#8fc8c9; }");
        this.writer.println("div.list table tr * {font-size:90%;}");
        this.writer.println("div.list table th, div.list table td {text-align:left;padding: 0; vertical-align:top;}");
        this.writer.println("div.list table th {padding-right:1em;width:0%;}");

        this.writer.println(".native {background-color:red !important;}");

        this.writer.println("</style>");

        this.writer.println("</head>");
        this.writer.println("<body>");
    }

    public void printInfo() {

        final Map<Object, Object> props = new HashMap<Object, Object>();
        props.put("Contacts", "<a href=\"http://diablitozzz.org\">http://diablitozzz.org</a>");
        this.printProps("Info", props, false);
    }

    public void printMemory() {

        double val;
        final NumberFormat formatter = new DecimalFormat("##,##,###");

        final Map<Object, Object> props = new HashMap<Object, Object>();

        val = Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory();
        props.put("Usage Memory", formatter.format(val) + " b");

        val = Runtime.getRuntime().freeMemory();
        props.put("Free Memory", formatter.format(val) + " b");

        val = Runtime.getRuntime().totalMemory();
        props.put("Total Memory", formatter.format(val) + " b");

        val = Runtime.getRuntime().maxMemory();
        props.put("Max Memory", formatter.format(val) + " b");

        this.printProps("Memory", props);
    }

    private void printProps(String title, Map<Object, Object> props) {
        this.printProps(title, props, true);
    }

    private void printProps(String title, Map<Object, Object> props, boolean escape) {

        this.writer.println("<div class=\"key-value exception\">");
        this.writer.println("<h2>" + title + "</h2>");
        this.writer.println("<table>");

        for (final Object k : props.keySet()) {

            if (props.get(k) == null) {
                continue;
            }

            String value = props.get(k).toString();
            String key = k.toString();
            if (escape) {
                key = StringEscapeUtils.escapeHtml(key.trim());
                value = StringEscapeUtils.escapeHtml(value.trim());
            }
            this.writer.println("<tr><th>" + key + "</th><td>" + value + "</td></tr>");
        }
        this.writer.println("</table>");
        this.writer.println("</div>");
    }

    public void printServletConfig(ServletConfig servletConfig) {

        final Map<Object, Object> props = new HashMap<Object, Object>();
        @SuppressWarnings("rawtypes")
        Enumeration keys;

        keys = servletConfig.getInitParameterNames();
        while (keys.hasMoreElements()) {
            final String key = keys.nextElement().toString();
            props.put(key, servletConfig.getInitParameter(key));
        }
        this.printProps("Servlet Config", props);
    }

    public void printServletContext(ServletContext servletContext) {

        final Map<Object, Object> props = new HashMap<Object, Object>();
        String value;
        @SuppressWarnings("rawtypes")
        Enumeration keys;

        // Servlet Context
        props.clear();
        props.put("Context Path", servletContext.getContextPath());
        props.put("Server Info", servletContext.getServerInfo());
        props.put("Servlet Context Name", servletContext.getServletContextName());
        value = servletContext.getMajorVersion() + "." + servletContext.getMinorVersion();
        props.put("Version", value);
        this.printProps("Servlet Context", props);

        // Servlet Attributes
        props.clear();
        keys = servletContext.getAttributeNames();
        while (keys.hasMoreElements()) {
            final String key = keys.nextElement().toString();
            props.put(key, servletContext.getAttribute(key));
        }
        this.printProps("Servlet Attributes", props);

        // Servlet Init parametrs
        props.clear();
        keys = servletContext.getInitParameterNames();
        while (keys.hasMoreElements()) {
            final String key = keys.nextElement().toString();
            props.put(key, servletContext.getInitParameter(key));
        }
        this.printProps("Servlet Init Parametrs", props);
    }

    public void printStackTrace(Throwable e) {

        String key, value, title;

        title = "Stack Trace";
        this.writer.println("<div class=\"list exception\">");
        this.writer.println("<h2>" + title + "</h2>");
        this.writer.println("<ul>");

        for (final StackTraceElement element : e.getStackTrace()) {

            if (element.isNativeMethod()) {
                this.writer.println("<li class=\"native\">");
            } else {
                this.writer.println("<li>");
            }

            this.writer.println("<table>");

            key = "Class:";
            value = element.getClassName();
            this.writer.println("<tr><th>" + key + "</th><td>" + value + "</td></tr>");

            key = "Method:";
            value = element.getMethodName();
            this.writer.println("<tr><th>" + key + "</th><td>" + value + "</td></tr>");

            key = "File:";
            value = element.getFileName();
            this.writer.println("<tr><th>" + key + "</th><td>" + value + "</td></tr>");

            key = "Line:";
            value = new Integer(element.getLineNumber()).toString();
            this.writer.println("<tr><th>" + key + "</th><td>" + value + "</td></tr>");

            this.writer.println("</table>");
            this.writer.println("</li>");
        }
        this.writer.println("</ul>");
        this.writer.println("</div>");

    }

    public void printSystemEnvironment() {
        final Map<Object, Object> props = new HashMap<Object, Object>();
        for (final String key : System.getenv().keySet()) {
            props.put(key, System.getenv(key));
        }
        this.printProps("System Environment", props);
    }

    public void printSystemProperties() {
        this.printProps("System Properties", System.getProperties());
    }

    public void setWriter(PrintWriter writer) {
        this.writer = writer;
    }
}
