package org.diablitozzz.jera.jetty.config;

import java.io.File;
import java.io.IOException;
import java.net.URISyntaxException;
import java.util.Map;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.junit.Assert;
import org.junit.Test;
import org.xml.sax.SAXException;

public class ConfigServiceTest {

    private void assertHostLocalhost(ConfigHost host)
    {
        Assert.assertEquals(host.getName(), "localhost");
        Assert.assertEquals(host.getRoot(), "/home/www/localhost/htdocs");
        Assert.assertNull(host.getBuild());
        Assert.assertNull(host.getParams());
    }

    private void assertHosts(ConfigHosts hosts)
    {
        File file = new File(hosts.getDir());
        Assert.assertTrue(file.exists());
        Assert.assertTrue(file.isDirectory());

        Assert.assertEquals(hosts.getHostList().size(), 2);

        for (ConfigHost host : hosts.getHostList()) {

            if (host.getName().equals("localhost")) {
                this.assertHostLocalhost(host);
            }
            else if (host.getName().equals("thumb.loc")) {
                this.assertHostThumbLoc(host);
            }
            else {
                Assert.fail();
            }
        }
    }

    private void assertHostThumbLoc(ConfigHost host)
    {
        Assert.assertEquals(host.getName(), "thumb.loc");
        Assert.assertEquals(host.getRoot(), "/home/diablitozzz/my/workspace/jera.thumbnail/target/webapp");

        // build
        ConfigBuild build = host.getBuild();
        Assert.assertNotNull(build);
        Assert.assertEquals(build.getAnt(), "/home/diablitozzz/my/workspace/jera.thumbnail/classes.build.xml");
        Assert.assertEquals(build.getScan(), "/home/diablitozzz/my/workspace/jera.thumbnail/src");
        Assert.assertEquals(build.getClasses(), "/home/diablitozzz/my/workspace/jera.thumbnail/target/webapp/WEB-INF/classes");

        // params
        Map<String, String> params = host.getParams();
        Assert.assertNotNull(params);
        Assert.assertEquals(params.size(), 2);

        Assert.assertEquals(params.get("config.url"), "http://localhost/image.ini");
        Assert.assertEquals(params.get("cache_marker"), "/home/my/workspace/jera.thumbnail/etc/image.ini");

    }

    private void assertServer(ConfigServer server)
    {
        Assert.assertEquals(server.getPort(), 3000);
        Assert.assertEquals(server.getErr(), "/tmp/jera.jetty.err.log");
        Assert.assertEquals(server.getOut(), "/tmp/jera.jetty.out.log");
        this.assertHosts(server.getHosts());
    }

    @Test
    public void testLoadConfig() throws ParserConfigurationException, SAXException, IOException, XPathExpressionException, URISyntaxException
    {
        ConfigService service = new ConfigService();
        ConfigServer server = service.loadConfig("config/config.xml");

        this.assertServer(server);
    }

}
