#!/bin/bash
### BEGIN INIT INFO
# Provides:          jera.jetty
# Required-Start:    $remote_fs $network
# Required-Stop:     $remote_fs $network
# Should-Start:      fam
# Should-Stop:       fam
# Default-Start:     2 3 4 5
# Default-Stop:      0 1 6
# Short-Description: Start the jera.jetty
### END INIT INFO

NAME=jera.jetty
DESC="jera.jetty"

CUR_FILE=`readlink -f $0`
CUR_DIR=`dirname $CUR_FILE`

#root
ROOT_DIR=`dirname "$CUR_DIR"`
ROOT_DIR=`dirname "$ROOT_DIR"` 
ROOT_DIR=`dirname "$ROOT_DIR"`
if [ $ROOT_DIR = "/" ]; then
    ROOT_DIR=""
fi

#etc
ETC_DIR="$ROOT_DIR/etc/jera.jetty"




#options
OPTIONS_FILE="$ETC_DIR/options"
source "$OPTIONS_FILE"


# java home
if [ -z "$JAVA_HOME"  ]; then
    echo "JAVA_HOME NOT SET"
    exit 1
fi
JAVA_BIN="$JAVA_HOME/bin/java"


# class path
LIB_DIR=$CUR_DIR
CLASS_PATH_FILE="$CUR_DIR/classpath"
source "$CLASS_PATH_FILE" 
CLASS_PATH="$CLASS_PATH:$JAVA_HOME/lib/tools.jar"




set -e
. /lib/lsb/init-functions


function info
{
  echo ""
  echo -e "CUR_DIR:		$CUR_DIR"
  echo -e "ETC_DIR:		$ETC_DIR"
  echo ""
  echo -e "CLASS_PATH_FILE:    \t$CLASS_PATH_FILE"
  echo -e "OPTIONS_FILE:	\t$OPTIONS_FILE"
  echo -e "CONFIG_FILE:		$CONFIG_FILE"
  echo ""
  echo -e "JSVC_BIN:		$JSVC_BIN"
  echo -e "JAVA_HOME:		$JAVA_HOME"
  echo -e "JAVA_BIN:		$JAVA_BIN"
  echo -e "JVM_OPTIONS:		$JVM_OPTIONS"  
  echo ""
  echo -e "OUT_FILE:		$OUT_FILE"
  echo -e "ERR_FILE:		$ERR_FILE"
  echo -e "PID_FILE:		$PID_FILE"
  echo -e "RUN_USER:		$RUN_USER"
  echo ""
  echo -e "CLI_CLASS:		$CLI_CLASS"
  echo -e "DAEMON_CLASS:	\t$DAEMON_CLASS"
  echo ""
  echo -e "CLASS_PATH:		$CLASS_PATH:"
  echo ""
}


function help
{
  echo ""
  echo -e "Usage:"
  echo -e "	start"
  echo -e "	start_debug"
  echo -e "	stop"
  echo -e "	stop_debug"
  echo -e "	restart"
  echo -e "	help"
  echo -e "	info"
  echo -e "	test"
  echo ""
}

function test
{
  sudo $JAVA_BIN $JVM_OPTIONS -classpath "$CLASS_PATH" $CLI_CLASS $CONFIG_FILE
}

function debug
{
  echo ""
  echo "OUT FILE:"
  sudo cat $OUT_FILE
  echo ""
  echo "ERR_FILE:"
  sudo cat $ERR_FILE
  echo ""
  echo "PID_FILE:"
  sudo cat $PID_FILE
  echo ""
  echo "PROCESES:"
  sudo  ps -A | grep jsvc
  echo ""
}

function restart
{
  stop
  sleep 1
  start
}

function start
{
  log_daemon_msg "Starting $DESC" $NAME

  if ! $JSVC_BIN \
	-user $RUN_USER \
	-home $JAVA_HOME \
	-cp $CLASS_PATH \
	-outfile $OUT_FILE \
	-errfile $ERR_FILE \
	-pidfile $PID_FILE \
	$JVM_OPTIONS \
	$DAEMON_CLASS $CONFIG_FILE
  then
    log_end_msg 1
  else
    log_end_msg 0
  fi
  
  chown $RUN_USER $OUT_FILE
  chown $RUN_USER $ERR_FILE
  
}

function start_debug
{
  sudo rm -rf $OUT_FILE
  sudo rm -rf $ERR_FILE
  start
  sleep 1
  debug
}

function stop_debug
{
  stop
  sleep 1
  debug
}

function stop
{
  log_daemon_msg "Stopping $DESC" $NAME
  if  $JSVC_BIN \
	-stop \
	-user $RUN_USER \
	-home $JAVA_HOME \
	-cp $CLASS_PATH \
	-outfile $OUT_FILE \
	-errfile $ERR_FILE \
	-pidfile $PID_FILE \
	$JVM_OPTIONS \
	$DAEMON_CLASS  $CONFIG_FILE
  then
    log_end_msg 0
  else
    log_end_msg 1
  fi
}

if [ -z "$1"  ]; then
  help
else
  $1
fi
